import { RuleConfigSeverity } from "@commitlint/types";
import commitTypes from "@gitmoji/commit-types";
import { emojiRegex, gitmojiUnicodeRegex, gitmojiCodeRegex } from "@gitmoji/gitmoji-regex";
import { gitmojis } from "gitmojis";
export const gitmojiCodes = gitmojis.map((gitmoji)=>gitmoji.code);
export const gitmojiUnicode = gitmojis.map((gitmoji)=>gitmoji.emoji);
const gitmojiCodeStr = gitmojiCodeRegex.source;
const gitmojiUnicodeStr = gitmojiUnicodeRegex.source;
const emojiStr = emojiRegex.source;
const { Error  } = RuleConfigSeverity;
const rules = {
    // 使用gitmoji 规则
    "with-gitmoji": [
        Error,
        "always"
    ],
    // 使用 Angular 的类型
    "type-enum": [
        Error,
        "always",
        commitTypes
    ],
    // 内容以空行开始
    "body-leading-blank": [
        Error,
        "always"
    ],
    // 结尾以空行开始
    "footer-leading-blank": [
        Error,
        "always"
    ],
    // 标题最大长度 100 个字符
    "header-max-length": [
        Error,
        "always",
        100
    ],
    // Scope 永远小写
    "scope-case": [
        Error,
        "always",
        "lower-case"
    ],
    // 不允许标题空着
    "subject-empty": [
        Error,
        "never"
    ],
    // 不允许使用句号
    "subject-full-stop": [
        Error,
        "never",
        "."
    ],
    // type 必须小写
    "type-case": [
        Error,
        "always",
        "lower-case"
    ],
    // type 不能为空
    "type-empty": [
        Error,
        "never"
    ]
};
const emojiRule = (parsed, _rule, value = "or-emoji")=>{
    // todo: 后续是否支持 emoji 在类型后面
    const { raw  } = parsed;
    // code regex test url: https://regex101.com/r/fSdOvB/1
    const gitmojiCodeResult = new RegExp(`(${gitmojiCodeRegex.source})\\s.*`, "gm").exec(raw);
    // unicode regex test url: https://regex101.com/r/shBTBg/2
    const gitmojiUnicodeResult = new RegExp(`(${gitmojiUnicodeRegex.source})\\s.*`, "gm").exec(raw);
    const emojiResult = new RegExp(`(${emojiRegex.source})\\s.*`, "gm").exec(raw);
    let pass;
    let errorMsg = "passed";
    // if gitmoji code is valid
    if (gitmojiCodeResult) {
        const emojiCode = gitmojiCodeResult[1];
        pass = gitmojiCodes.includes(emojiCode);
        if (!pass) {
            errorMsg = `${emojiCode} is not in the correct gitmoji list, please check the emoji code on https://gitmoji.dev/.`;
        }
    } else if (gitmojiUnicodeResult) {
        const unicode = gitmojiUnicodeResult[1];
        pass = gitmojiUnicode.includes(unicode);
    } else if (emojiResult) {
        const unicode = emojiResult[1];
        pass = false;
        errorMsg = `${unicode} is not in the correct gitmoji list, please check the emoji code on https://gitmoji.dev/.`;
    } else {
        // 没有 emoji 通过
        if (value === "start-emoji") {
            pass = false;
            errorMsg = "Your commit should start with gitmoji code. Please check the emoji code on https://gitmoji.dev/.";
        } else {
            pass = true;
        }
    }
    return [
        pass,
        errorMsg
    ];
};
const gitmojiPlugin = {
    rules: {
        "with-gitmoji": emojiRule
    }
};
const parserPreset = {
    parserOpts: {
        headerPattern: new RegExp(`^(?:${gitmojiCodeStr}|(?:${gitmojiUnicodeStr})|(?:${emojiStr}))\\s(?<type>\\w*)(?:\\((?<scope>.*)\\))?!?:\\s(?<subject>(?:(?!#).)*(?:(?!\\s).))(?:\\s\\(?(?<ticket>#\\d*)\\)?)?$`),
        headerCorrespondence: [
            "type",
            "scope",
            "subject",
            "ticket"
        ]
    }
};
const config = {
    rules,
    parserPreset,
    plugins: [
        gitmojiPlugin
    ]
};
export default config;
